@@ -18,7 +18,7 @@ from glob import glob
 import os, subprocess, shutil, sys, signal
 from io import StringIO
 from ast import literal_eval
-import sys, tempfile
+import tempfile
 import mesontest
 from mesonbuild import environment
 from mesonbuild import mesonlib
@@ -43,12 +43,14 @@ class TestResult:
         self.buildtime = buildtime
         self.testtime = testtime
 
-class AutoDeletedDir():
+class AutoDeletedDir:
     def __init__(self, d):
         self.dir = d
+
     def __enter__(self):
         os.makedirs(self.dir, exist_ok=True)
         return self.dir
+
     def __exit__(self, _type, value, traceback):
         # On Windows, shutil.rmtree fails sometimes, because 'the directory is not empty'.
         # Retrying fixes this.
@@ -58,9 +60,15 @@ class AutoDeletedDir():
             try:
                 shutil.rmtree(self.dir)
                 return
-            except OSError:
-                if i == retries-1:
-                    raise
+            # Sometimes we get: ValueError: I/O operation on closed file.
+            except ValueError:
+                return
+            # Deleting can raise OSError or PermissionError on Windows
+            # (most likely because of anti-virus locking the file)
+            except (OSError, PermissionError):
+                if i == retries - 1:
+                    mlog.warning('Could not delete temporary directory.')
+                    return
                 time.sleep(0.1 * (2**i))
 
 failing_logs = []
@@ -84,7 +92,7 @@ def stop_handler(signal, frame):
 signal.signal(signal.SIGINT, stop_handler)
 signal.signal(signal.SIGTERM, stop_handler)
 
-#unity_flags = ['--unity']
+# unity_flags = ['--unity']
 unity_flags = []
 
 backend_flags = None
@@ -97,6 +105,8 @@ def setup_commands(backend):
     global backend_flags, compile_commands, test_commands, install_commands, clean_commands
     msbuild_exe = shutil.which('msbuild')
     if (backend and backend.startswith('vs')) or (backend is None and msbuild_exe is not None):
+        if backend is None:
+            backend = 'vs2010'
         backend_flags = ['--backend=' + backend]
         compile_commands = ['msbuild']
         test_commands = ['msbuild', 'RUN_TESTS.vcxproj']
@@ -106,7 +116,8 @@ def setup_commands(backend):
         test_commands = ['xcodebuild', '-target', 'RUN_TESTS']
     else:
         backend_flags = []
-        ninja_command = environment.detect_ninja()
+        # We need at least 1.6 because of -w dupbuild=err
+        ninja_command = environment.detect_ninja(version='1.6')
         if ninja_command is None:
             raise RuntimeError('Could not find Ninja v1.6 or newer')
         if do_debug:
@@ -118,6 +129,14 @@ def setup_commands(backend):
         install_commands = [ninja_command, 'install']
         clean_commands = [ninja_command, 'clean']
 
+def get_compile_commands_for_dir(compile_commands, test_build_dir):
+    if 'msbuild' in compile_commands[0]:
+        sln_name = glob(os.path.join(test_build_dir, '*.sln'))[0]
+        comp = compile_commands + [os.path.split(sln_name)[-1]]
+    else:
+        comp = compile_commands
+    return comp
+
 def get_relative_files_list_from_dir(fromdir):
     paths = []
     for (root, _, files) in os.walk(fromdir):
@@ -193,7 +212,7 @@ def run_configure_inprocess(commandlist):
     finally:
         sys.stdout = old_stdout
         sys.stderr = old_stderr
-    return (returncode, mystdout.getvalue(), mystderr.getvalue())
+    return returncode, mystdout.getvalue(), mystderr.getvalue()
 
 def run_test_inprocess(testdir):
     old_stdout = sys.stdout
@@ -209,7 +228,7 @@ def run_test_inprocess(testdir):
         sys.stdout = old_stdout
         sys.stderr = old_stderr
         os.chdir(old_cwd)
-    return (max(returncode_test, returncode_benchmark), mystdout.getvalue(), mystderr.getvalue())
+    return max(returncode_test, returncode_benchmark), mystdout.getvalue(), mystderr.getvalue()
 
 def parse_test_args(testdir):
     args = []
@@ -239,6 +258,7 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
     global install_commands, clean_commands
     test_args = parse_test_args(testdir)
     gen_start = time.time()
+    # Configure in-process
     gen_command = [meson_command, '--prefix', '/usr', '--libdir', 'lib', testdir, test_build_dir]\
         + flags + test_args + extra_args
     (returncode, stdo, stde) = run_configure_inprocess(gen_command)
@@ -255,11 +275,8 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
         return TestResult('Test that should have failed succeeded', stdo, stde, mesonlog, gen_time)
     if returncode != 0:
         return TestResult('Generating the build system failed.', stdo, stde, mesonlog, gen_time)
-    if 'msbuild' in compile_commands[0]:
-        sln_name = glob(os.path.join(test_build_dir, '*.sln'))[0]
-        comp = compile_commands + [os.path.split(sln_name)[-1]]
-    else:
-        comp = compile_commands
+    # Build with subprocess
+    comp = get_compile_commands_for_dir(compile_commands, test_build_dir)
     build_start = time.time()
     pc, o, e = Popen_safe(comp, cwd=test_build_dir)
     build_time = time.time() - build_start
@@ -271,10 +288,13 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
         return TestResult('Test that should have failed to build succeeded', stdo, stde, mesonlog, gen_time)
     if pc.returncode != 0:
         return TestResult('Compiling source code failed.', stdo, stde, mesonlog, gen_time, build_time)
+    # Touch the meson.build file to force a regenerate so we can test that
+    # regeneration works. We need to sleep for 0.2s because Ninja tracks mtimes
+    # at a low resolution: https://github.com/ninja-build/ninja/issues/371
+    time.sleep(0.2)
+    os.utime(os.path.join(testdir, 'meson.build'))
     test_start = time.time()
-    # Note that we don't test that running e.g. 'ninja test' actually
-    # works. One hopes that this is a common enough happening that
-    # it is picked up immediately on development.
+    # Test in-process
     (returncode, tstdo, tstde) = run_test_inprocess(test_build_dir)
     test_time = time.time() - test_start
     stdo += tstdo
@@ -285,11 +305,11 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
         return TestResult('Test that should have failed to run unit tests succeeded', stdo, stde, mesonlog, gen_time)
     if returncode != 0:
         return TestResult('Running unit tests failed.', stdo, stde, mesonlog, gen_time, build_time, test_time)
-    # Do installation
     if len(install_commands) == 0:
         return TestResult('', '', '', gen_time, build_time, test_time)
     env = os.environ.copy()
     env['DESTDIR'] = install_dir
+    # Install with subprocess
     pi, o, e = Popen_safe(install_commands, cwd=test_build_dir, env=env)
     stdo += o
     stde += e
@@ -297,6 +317,7 @@ def _run_test(testdir, test_build_dir, install_dir, extra_args, flags, compile_c
         return TestResult('Running install failed.', stdo, stde, mesonlog, gen_time, build_time, test_time)
     if len(clean_commands) != 0:
         env = os.environ.copy()
+        # Clean with subprocess
         pi, o, e = Popen_safe(clean_commands, cwd=test_build_dir, env=env)
         stdo += o
         stde += e
@@ -390,7 +411,7 @@ def run_tests(all_tests, log_name_base, extra_args):
     executor = conc.ProcessPoolExecutor(max_workers=num_workers)
 
     for name, test_cases, skipped in all_tests:
-        current_suite = ET.SubElement(junit_root, 'testsuite', {'name' : name, 'tests' : str(len(test_cases))})
+        current_suite = ET.SubElement(junit_root, 'testsuite', {'name': name, 'tests': str(len(test_cases))})
         if skipped:
             print('\nNot running %s tests.\n' % name)
         else:
@@ -410,8 +431,8 @@ def run_tests(all_tests, log_name_base, extra_args):
             result = result.result()
             if result is None or 'MESON_SKIP_TEST' in result.stdo:
                 print('Skipping:', t)
-                current_test = ET.SubElement(current_suite, 'testcase', {'name' : testname,
-                                                                         'classname' : name})
+                current_test = ET.SubElement(current_suite, 'testcase', {'name': testname,
+                                                                         'classname': name})
                 ET.SubElement(current_test, 'skipped', {})
                 skipped_tests += 1
             else:
@@ -430,11 +451,11 @@ def run_tests(all_tests, log_name_base, extra_args):
                 test_time += result.testtime
                 total_time = conf_time + build_time + test_time
                 log_text_file(logfile, t, result.stdo, result.stde)
-                current_test = ET.SubElement(current_suite, 'testcase', {'name' : testname,
-                                                                         'classname' : name,
-                                                                         'time' : '%.3f' % total_time})
+                current_test = ET.SubElement(current_suite, 'testcase', {'name': testname,
+                                                                         'classname': name,
+                                                                         'time': '%.3f' % total_time})
                 if result.msg != '':
-                    ET.SubElement(current_test, 'failure', {'message' : result.msg})
+                    ET.SubElement(current_test, 'failure', {'message': result.msg})
                 stdoel = ET.SubElement(current_test, 'system-out')
                 stdoel.text = result.stdo
                 stdeel = ET.SubElement(current_test, 'system-err')
@@ -443,7 +464,7 @@ def run_tests(all_tests, log_name_base, extra_args):
     print("Total build time: %.2fs" % build_time)
     print("Total test time: %.2fs" % test_time)
     ET.ElementTree(element=junit_root).write(xmlname, xml_declaration=True, encoding='UTF-8')
-    return (passing_tests, failing_tests, skipped_tests)
+    return passing_tests, failing_tests, skipped_tests
 
 def check_file(fname):
     linenum = 1
@@ -467,7 +488,7 @@ def check_format():
 
 def pbcompile(compiler, source, objectfile):
     if compiler == 'cl':
-        cmd = [compiler, '/nologo', '/Fo'+objectfile, '/c', source]
+        cmd = [compiler, '/nologo', '/Fo' + objectfile, '/c', source]
     else:
         cmd = [compiler, '-c', source, '-o', objectfile]
     subprocess.check_call(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
@@ -508,14 +529,38 @@ def generate_prebuilt():
         object_suffix = 'o'
     objectfile = generate_pb_object(compiler, object_suffix)
     stlibfile = generate_pb_static(compiler, object_suffix, static_suffix)
-    return (objectfile, stlibfile)
+    return objectfile, stlibfile
+
+def check_meson_commands_work():
+    global meson_command, compile_commands, test_commands, install_commands
+    testdir = 'test cases/common/1 trivial'
+    with AutoDeletedDir(tempfile.mkdtemp(prefix='b ', dir='.')) as build_dir:
+        print('Checking that configuring works...')
+        gen_cmd = [sys.executable, meson_command, testdir, build_dir] + backend_flags
+        pc, o, e = Popen_safe(gen_cmd)
+        if pc.returncode != 0:
+            raise RuntimeError('Failed to configure {!r}:\n{}\n{}'.format(testdir, e, o))
+        print('Checking that building works...')
+        compile_cmd = get_compile_commands_for_dir(compile_commands, build_dir)
+        pc, o, e = Popen_safe(compile_cmd, cwd=build_dir)
+        if pc.returncode != 0:
+            raise RuntimeError('Failed to build {!r}:\n{}\n{}'.format(testdir, e, o))
+        print('Checking that testing works...')
+        pc, o, e = Popen_safe(test_commands, cwd=build_dir)
+        if pc.returncode != 0:
+            raise RuntimeError('Failed to test {!r}:\n{}\n{}'.format(testdir, e, o))
+        if install_commands:
+            print('Checking that installing works...')
+            pc, o, e = Popen_safe(install_commands, cwd=build_dir)
+            if pc.returncode != 0:
+                raise RuntimeError('Failed to install {!r}:\n{}\n{}'.format(testdir, e, o))
 
 if __name__ == '__main__':
     parser = argparse.ArgumentParser(description="Run the test suite of Meson.")
     parser.add_argument('extra_args', nargs='*',
-                   help='arguments that are passed directly to Meson (remember to have -- before these).')
+                        help='arguments that are passed directly to Meson (remember to have -- before these).')
     parser.add_argument('--backend', default=None, dest='backend',
-                        choices = backendlist)
+                        choices=backendlist)
     options = parser.parse_args()
     setup_commands(options.backend)
 
@@ -538,6 +583,7 @@ if __name__ == '__main__':
     if script_dir != '':
         os.chdir(script_dir)
     check_format()
+    check_meson_commands_work()
     pbfiles = generate_prebuilt()
     try:
         all_tests = detect_tests_to_run()
@@ -549,9 +595,8 @@ if __name__ == '__main__':
     print('\nTotal passed tests:', passing_tests)
     print('Total failed tests:', failing_tests)
     print('Total skipped tests:', skipped_tests)
-    if failing_tests > 0 and ('TRAVIS' in os.environ or 'APPVEYOR' in os.environ):
+    if failing_tests > 0:
         print('\nMesonlogs of failing tests\n')
         for l in failing_logs:
             print(l, '\n')
     sys.exit(failing_tests)
-
