@@ -14,14 +14,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import subprocess, sys, os
-import shutil
+import subprocess, sys, shutil
+import platform
 from mesonbuild import mesonlib
 
 if __name__ == '__main__':
     returncode = 0
+    print('Running unittests.\n')
     if mesonlib.is_linux():
-        print('Running unittests.\n')
         returncode += subprocess.call([sys.executable, 'run_unittests.py', '-v'])
+    else:
+        returncode += subprocess.call([sys.executable, 'run_unittests.py', '-v', 'InternalTests'])
+    # Ubuntu packages do not have a binary without -6 suffix.
+    if shutil.which('arm-linux-gnueabihf-gcc-6') and not platform.machine().startswith('arm'):
+        print('Running cross compilation tests.\n')
+        returncode += subprocess.call([sys.executable, 'run_cross_test.py', 'cross/ubuntu-armhf.txt'])
     returncode += subprocess.call([sys.executable, 'run_project_tests.py'] + sys.argv[1:])
     sys.exit(returncode)
